from collections import defaultdict

# Project directories
BASE_DIR = './aaransia'
DATA_DIR = '/data'
LOG_DIR = '/log'
TEST_DIR = '/test'

# Data sets
CONTROLLED_DICTIONARY = '/controlled_dictionary.csv'
DICTIONARY = '/dictionary.csv'
DICTIONARY_SAMPLE = '/dictionary_sample.csv'
TRANING_DATA = '/training_data.csv'
ALPHABET = '/alphabet.csv'

# Test files
TEST_CASES = '/test_cases.csv'
TEST_ALPHABET_MA_AR = '/test_alphabet_ma_ar.csv'
TEST_WORDS_MA_AR = '/test_words_ma_ar.csv'

TEST_ALPHABET_AR_MA = '/test_alphabet_ar_ma.csv'
TEST_WORDS_AR_MA = '/test_words_ar_ma.csv'

# Test logs files
TEST_CASE_LOG_FILE = '/test_case.log'

ALPHABET_MA_AR_TEST_LOG_FILE = '/test_alphabet_ma_ar.log'
WORD_MA_AR_TEST_LOG_FILE = '/test_word_ma_ar.log'
SENTENCE_MA_AR_TEST_LOG_FILE = '/test_sentence_ma_ar.log'

ALPHABET_AR_MA_TEST_LOG_FILE = '/test_alphabet_ar_ma.log'
WORD_AR_MA_TEST_LOG_FILE = '/test_word_ar_ma.log'
SENTENCE_AR_MA_TEST_LOG_FILE = '/test_sentence_ar_ma.log'

ARABIC_TRANSLATION_TEST_LOG_FILE = '/test_arabic_translation.log'
FRENCH_TRANSLATION_TEST_LOG_FILE = '/test_french_translation.log'
ENGLISH_TRANSLATION_TEST_LOG_FILE = '/test_english_translation.log'

TEST_STATS_LOG_FILE = '/test_stats.log'

# Double moroccan letters
DOUBLE_MOROCCAN_LETTERS = ['la', 'kh', 'sh', 'ou', 'gh', 'ch', 'yi', 'ee']

# Alphabets
moroccan_alphabet = {   
                ' ': [' '],
                '!': ['!'],
                ',': ['،'],
                '.': ['.'],
                '2': ['ء'],
                '3': ['ع'],
                '5': ['خ'],
                '7': ['ح'],
                '9': ['ق'],
                '?': ['؟'],
                'a': ['ا', 'أ'],
                'b': ['ب'],
                'c': ['ك'],
                'ch': ['ش'],
                'd': ['د', 'ذ', 'ض', 'ظ'],
                'e': [' ', 'إ'],
                'ee': ['ي'],
                'f': ['ف'],
                'g': ['ڭ'],
                'gh': ['غ'],
                'h': ['ه'],
                'i': ['ي', 'إ'],
                'j': ['ج'],
                'k': ['ك'],
                'kh': ['خ'],
                'l': ['ل'],
                'la': ['لا'],
                'm': ['م'],
                'n': ['ن'],
                'o': ['و', 'أ'],
                'ou': ['و'],
                'p': ['پ'],
                'q': ['ق'],
                'r': ['ر'],
                's': ['س', 'ص'],
                'sh': ['ش'],
                't': ['ت', 'ة', 'ط', 'ث'],
                'u': ['و'],
                'v': ['ڤ'],
                'w': ['و'],
                'x': ['كس', 'كز'],
                'y': ['ي'],
                'yi': ['ي'],
                'z': ['ز']}

moroccan_arabic_alphabet = {   
                ' ': [' ', 'e'],
                '!': ['!'],
                '.': ['.'],
                'Test Case': ['Expected Result'],
                '،': [','],
                '؟': ['?'],
                'ء': ['2'],
                'أ': ['o', 'a'],
                'إ': ['e', 'i'],
                'ا': ['a'],
                'ب': ['b'],
                'ة': ['t'],
                'ت': ['t'],
                'ث': ['t'],
                'ج': ['j'],
                'ح': ['7'],
                'خ': ['5', 'kh'],
                'د': ['d'],
                'ذ': ['d'],
                'ر': ['r'],
                'ز': ['z'],
                'س': ['s'],
                'ش': ['ch', 'sh'],
                'ص': ['s'],
                'ض': ['d'],
                'ط': ['t'],
                'ظ': ['d'],
                'ع': ['3'],
                'غ': ['gh'],
                'ف': ['f'],
                'ق': ['9', 'q'],
                'ك': ['k', 'c'],
                'كز': ['x'],
                'كس': ['x'],
                'ل': ['l'],
                'لا': ['la'],
                'م': ['m'],
                'ن': ['n'],
                'ه': ['h'],
                'و': ['o', 'w', 'ou', 'u'],
                'ي': ['i', 'y', 'yi', 'ee'],
                'پ': ['p'],
                'ڤ': ['v'],
                'ڭ': ['g']}


import sys
import operator

import pandas as pd
import numpy as np
import nltk

from aaransia.constants import *
from aaransia.utils import *

traning_data = pd.read_csv(BASE_DIR + DATA_DIR + TRANING_DATA)

# Model to guess the moroccan L/D

## Tokenization
moroccan_words_tokens_x = {word : i for (i, word) in enumerate(traning_data["Moroccan"])}
moroccan_words_tokens_y = {word : i for (i, word) in enumerate(traning_data["Moroccan Arabic"])}

# # Padding 
# measurer = np.vectorize(len)
# max_length_x = measurer(moroccan_words_tokens_x.values.astype(str)).max(axis=0)



# Function to run the machine leatning algorithms
def run_machine_learning():
    # print(get_tokens("oustouraliya almaniya", moroccan_tokens))
    print(max(moroccan_words_tokens_x.items(), key=lambda x:len(x)))
    print(max(moroccan_words_tokens_y.items(), key=lambda y:len(y)))



import re
import csv
import collections
import pprint

import pandas as pd

from aaransia.constants import *

pp = pprint.PrettyPrinter(indent=4)

# Build test moroccan alphabet file
def build_test_alphabet_ma_ar_file():
    alphabet = pd.read_csv(BASE_DIR + DATA_DIR + ALPHABET)
    alphabet_test = alphabet.rename(columns={
                          'Moroccan Alphabet':'Test Case',
                          'Arabian Alphabet':'Expected Result'})
    alphabet_test = alphabet_test[['Test Case', 'Expected Result']]
    alphabet_test.to_csv(BASE_DIR + TEST_DIR + TEST_ALPHABET_MA_AR, index=False)

# Build Test moroccan words File
def build_test_words_ma_ar_file():
    moroccan_words = pd.read_csv(BASE_DIR + DATA_DIR + DICTIONARY_SAMPLE)
    moroccan_words_test = moroccan_words.drop(columns=['Arabic','French', 'English'])
    moroccan_words_test = moroccan_words_test.rename(columns={
                          'Moroccan':'Test Case',
                          'Moroccan Arabic':'Expected Result'})
    moroccan_words_test = moroccan_words_test[['Test Case', 'Expected Result']]
    moroccan_words_test.to_csv(BASE_DIR + TEST_DIR + TEST_WORDS_MA_AR, index=False)

# Build test arabian alphabet file
def build_test_alphabet_ar_ma_file():
    alphabet = pd.read_csv(BASE_DIR + DATA_DIR + ALPHABET)
    alphabet_test = alphabet.rename(columns={
                          'Arabian Alphabet':'Test Case',
                          'Moroccan Alphabet':'Expected Result'})
    alphabet_test = alphabet_test[['Test Case', 'Expected Result']]
    alphabet_test.to_csv(BASE_DIR + TEST_DIR + TEST_ALPHABET_AR_MA, index=False)

# Build Test arabian words File
def build_test_words_ar_ma_file():
    moroccan_words = pd.read_csv(BASE_DIR + DATA_DIR + DICTIONARY_SAMPLE)
    moroccan_words_test = moroccan_words.drop(columns=['Arabic','French', 'English'])
    moroccan_words_test = moroccan_words_test.rename(columns={
                          'Moroccan Arabic':'Test Case',
                          'Moroccan':'Expected Result'})
    moroccan_words_test = moroccan_words_test[['Test Case', 'Expected Result']]
    moroccan_words_test.to_csv(BASE_DIR + TEST_DIR + TEST_WORDS_AR_MA, index=False)

# Remove noise from arabic text
def de_noise(text):
    noise = re.compile(""" ّ    | # Tashdid
                             َ    | # Fatha
                             ً    | # Tanwin Fath
                             ُ    | # Damma
                             ٌ    | # Tanwin Damm
                             ِ    | # Kasra
                             ٍ    | # Tanwin Kasr
                             ْ    | # Sukun
                             ـ     # Tatwil/Kashida
                         """, re.VERBOSE)
    text = re.sub(noise, '', text)
    return text

# Normalize arabic
def normalize_arabic(text):
    text = re.sub("[إأٱآا]", "ا", text)
    text = re.sub("ى", "ي", text)
    text = re.sub("ؤ", "ء", text)
    text = re.sub("ئ", "ء", text)
    return(text)

# Construct the moroccan alphabet as defaultdict
def construct_moroccan_alphabet():
    with open(BASE_DIR + DATA_DIR + ALPHABET,'r') as f:
        r = csv.reader(f)
        dd = collections.defaultdict(list)
        for row in r:
            dd[row[0]].append(row[1])
    pp.pprint(dd)

# Construct the moroccan arabic alphabet as defaultdict
def construct_moroccan_arabic_alphabet():
    with open(BASE_DIR + TEST_DIR + TEST_ALPHABET_AR_MA,'r') as f:
        r = csv.reader(f)
        dd = collections.defaultdict(list)
        for row in r:
            dd[row[0]].append(row[1])
    pp.pprint(dd)


import logging
import logging.handlers
import os

from aaransia.transliterator import *
from aaransia.constants import *
from aaransia.machine_learning import *
from aaransia.utils import *

# construct_moroccan_alphabet()
# construct_moroccan_arabic_alphabet()

# Refresh test files
build_test_alphabet_ma_ar_file()
build_test_words_ma_ar_file()

build_test_alphabet_ar_ma_file()
build_test_words_ar_ma_file()

# Logging config
logging.root.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# Loggers
test_stats_logger = logging.getLogger('test_stats_logger')
test_case_logger = logging.getLogger('test_case_logger')

alphabet_ma_ar_logger = logging.getLogger('alphabet_ma_ar_logger')
word_ma_ar_logger = logging.getLogger('word_ma_ar_logger')

alphabet_ar_ma_logger = logging.getLogger('alphabet_ar_ma_logger')
word_ar_ma_logger = logging.getLogger('word_ar_ma_logger')

sentence_logger = logging.getLogger('sentence_logger')
arabic_translation_logger = logging.getLogger('arabic_translation_logger')
french_translation_logger = logging.getLogger('french_translation_logger')
english_translation_logger = logging.getLogger('english_translation_logger')

# Test sets
test_cases = pd.read_csv(BASE_DIR + TEST_DIR + TEST_CASES)

alphabet_ma_ar = pd.read_csv(BASE_DIR + TEST_DIR + TEST_ALPHABET_MA_AR)
words_ma_ar = pd.read_csv(BASE_DIR + TEST_DIR + TEST_WORDS_MA_AR)

alphabet_ar_ma = pd.read_csv(BASE_DIR + TEST_DIR + TEST_ALPHABET_AR_MA)
words_ar_ma = pd.read_csv(BASE_DIR + TEST_DIR + TEST_WORDS_AR_MA)

# Test statistics logger
fh = logging.FileHandler(BASE_DIR + LOG_DIR + TEST_STATS_LOG_FILE, 'a')
fh.setFormatter(formatter)
fh.setLevel(logging.INFO)
test_stats_logger.addHandler(fh)
test_stats_logger.info(f'\n')

# Test case 
def test_case():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + TEST_CASE_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    test_case_logger.addHandler(fh)
    test_case_logger.info(f'Translating [item to test] ([expected result], [generated result result])')

    count_infos, count_warnings, count_errors = 0, 0, 0

    for index, row in test_cases.iterrows():
        expected_result, moroccan_translation = row["Expected Result"], moroccan_to_arabic(row["Test Case"])
        try:
            if expected_result == moroccan_translation:
                test_case_logger.info(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_infos += 1
            else:
                test_case_logger.warning(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_warnings += 1
        except (IndexError, KeyError, TypeError) as e:
            test_case_logger.error(f'Translating {row["Test Case"]}, {e}')
            count_errors += 1
    test_case_logger.info(f'Total INFO logs {count_infos} ({round(count_infos/len(test_cases)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(test_cases)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(test_cases)*100, 2)}%)')
    test_stats_logger.info(f'test_case_logger - Total INFO logs {count_infos} ({round(count_infos/len(test_cases)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(test_cases)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(test_cases)*100, 2)}%)')


# Test moroccan alphabet
def test_alphabet_ma_ar_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + ALPHABET_MA_AR_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    alphabet_ma_ar_logger.addHandler(fh)
    alphabet_ma_ar_logger.info(f'Translating [item to test] ([expected result], [generated result result])')

    count_infos, count_warnings, count_errors = 0, 0, 0

    for index, row in alphabet_ma_ar.iterrows():
        expected_result, moroccan_translation = row["Expected Result"], moroccan_to_arabic(row["Test Case"])
        try:
            if expected_result == moroccan_translation:
                alphabet_ma_ar_logger.info(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_infos += 1
            else:
                alphabet_ma_ar_logger.warning(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_warnings += 1
        except (IndexError, KeyError, TypeError) as e:
            alphabet_ma_ar_logger.error(f'Translating {row["Test Case"]}, {e}')
            count_errors += 1
    alphabet_ma_ar_logger.info(f'Total INFO logs {count_infos} ({round(count_infos/len(alphabet_ma_ar)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(alphabet_ma_ar)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(alphabet_ma_ar)*100, 2)}%)')
    test_stats_logger.info(f'alphabet_ma_ar_logger - Total INFO logs {count_infos} ({round(count_infos/len(alphabet_ma_ar)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(alphabet_ma_ar)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(alphabet_ma_ar)*100, 2)}%)')

# Test moroccan words
def test_word_ma_ar_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + WORD_MA_AR_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    word_ma_ar_logger.addHandler(fh)
    word_ma_ar_logger.info(f'Translating [item to test] ([expected result], [generated result result])')

    count_infos, count_warnings, count_errors = 0, 0, 0

    for index, row in words_ma_ar.iterrows():
        expected_result, moroccan_translation = row["Expected Result"], moroccan_to_arabic(row["Test Case"])
        try:
            if expected_result == moroccan_translation:
                word_ma_ar_logger.info(f'Translating {row["Test Case"]} ({expected_result} , {moroccan_translation})')
                count_infos += 1
            else:
                word_ma_ar_logger.warning(f'Translating {row["Test Case"]} ({expected_result} , {moroccan_translation})')
                count_warnings += 1
        except (IndexError, KeyError, TypeError) as e:
            alphabet_ma_ar_logger.error(f'Translating {row["Test Case"]}, {e}')
            count_errors += 1
    word_ma_ar_logger.info(f'Total INFO logs {count_infos} ({round(count_infos/len(words_ma_ar)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(words_ma_ar)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(words_ma_ar)*100, 2)}%)')
    test_stats_logger.info(f'word_ma_ar_logger - Total INFO logs {count_infos} ({round(count_infos/len(words_ma_ar)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(words_ma_ar)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(words_ma_ar)*100, 2)}%)')

# Test arabian moroccan alphabet
def test_alphabet_ar_ma_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + ALPHABET_AR_MA_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    alphabet_ar_ma_logger.addHandler(fh)
    alphabet_ar_ma_logger.info(f'Translating [item to test] ([expected result], [generated result result])')

    count_infos, count_warnings, count_errors = 0, 0, 0

    for index, row in alphabet_ar_ma.iterrows():
        expected_result, moroccan_translation = row["Expected Result"], moroccan_arabic_to_moroccan(row["Test Case"])
        try:
            if expected_result == moroccan_translation:
                alphabet_ar_ma_logger.info(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_infos += 1
            else:
                alphabet_ar_ma_logger.warning(f'Translating {row["Test Case"]} ({expected_result}, {moroccan_translation})')
                count_warnings += 1
        except (IndexError, KeyError, TypeError) as e:
            alphabet_ar_ma_logger.error(f'Translating {row["Test Case"]}, {e}')
            count_errors += 1
    alphabet_ar_ma_logger.info(f'Total INFO logs {count_infos} ({round(count_infos/len(alphabet_ar_ma)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(alphabet_ar_ma)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(alphabet_ar_ma)*100, 2)}%)')
    test_stats_logger.info(f'alphabet_ar_ma_logger - Total INFO logs {count_infos} ({round(count_infos/len(alphabet_ar_ma)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(alphabet_ar_ma)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(alphabet_ar_ma)*100, 2)}%)')

# Test arabian moroccan words
def test_word_ar_ma_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + WORD_AR_MA_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    word_ar_ma_logger.addHandler(fh)
    word_ar_ma_logger.info(f'Translating [item to test] ([expected result], [generated result result])')

    count_infos, count_warnings, count_errors = 0, 0, 0

    for index, row in words_ar_ma.iterrows():
        expected_result, moroccan_translation = row["Expected Result"], moroccan_arabic_to_moroccan(row["Test Case"])
        try:
            if expected_result.lower() == moroccan_translation:
                word_ar_ma_logger.info(f'Translating {row["Test Case"]} ({expected_result} , {moroccan_translation})')
                count_infos += 1
            else:
                word_ar_ma_logger.warning(f'Translating {row["Test Case"]} ({expected_result} , {moroccan_translation})')
                count_warnings += 1
        except (IndexError, KeyError, TypeError) as e:
            alphabet_ar_ma_logger.error(f'Translating {row["Test Case"]}, {e}')
            count_errors += 1
    word_ar_ma_logger.info(f'Total INFO logs {count_infos} ({round(count_infos/len(words_ar_ma)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(words_ar_ma)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(words_ar_ma)*100, 2)}%)')
    test_stats_logger.info(f'word_ar_ma_logger - Total INFO logs {count_infos} ({round(count_infos/len(words_ar_ma)*100, 2)}%), Total WARNING logs {count_warnings} ({round(count_warnings/len(words_ar_ma)*100, )}%), Total ERROR logs {count_errors} ({round(count_errors/len(words_ar_ma)*100, 2)}%)')

# Test moroccan sentences
def test_sentence_ma_ar_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + SENTENCE_MA_AR_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    sentence_logger.addHandler(fh)

# Test arabian moroccan sentences
def test_sentence_ar_ma_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + SENTENCE_AR_MA_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    sentence_logger.addHandler(fh)

# Test Arabic translation
def test_arabic_translation_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + ARABIC_TRANSLATION_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    arabic_translation_logger.addHandler(fh)

# Test French translation
def test_french_translation_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + FRENCH_TRANSLATION_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    french_translation_logger.addHandler(fh)

# Test English translation
def test_english_translation_translation():
    fh = logging.FileHandler(BASE_DIR + LOG_DIR + ENGLISH_TRANSLATION_TEST_LOG_FILE, 'w')
    fh.setFormatter(formatter)
    fh.setLevel(logging.INFO)
    english_translation_logger.addHandler(fh)

# Test function
def run_tests():
    # Test case
    test_case()

    # Test morrocan alphabet to arabic
    test_alphabet_ma_ar_translation()
    
    # Test moroccan words to arabic
    test_word_ma_ar_translation()
    
    # Test moroccan sentenses to arabic
    test_sentence_ma_ar_translation()

    # Test arabic alphabet to moroccan
    test_alphabet_ar_ma_translation()
    
    # Test arabic words to moroccan
    test_word_ar_ma_translation()
    
    # Test arabic sentenses to moroccan
    test_sentence_ar_ma_translation()

    # Test Arabic translation of words
    test_arabic_translation_translation()
    
    # Test French translation of words
    test_french_translation_translation()

    # Test English translation of words
    test_english_translation_translation() 

    






import re

import unidecode
import pandas as pd
import numpy as np

from aaransia.constants import *
from aaransia.utils import *

# Transliterate a Moroccan letter to an Arabian letter
def _morrocan_letter_to_arabian(letter, position, word): 
    if ((letter in 'o'  and len(word) > 1) or letter in ('a', 'ou')) and position == 0: return 'أ'
    elif letter in ('i', 'e') and position == 0: return 'إ'
    elif letter == 'e' and position > 0: return ''
    else: return moroccan_alphabet[letter.lower()][0]

# Translate Moroccan to Arabic
def moroccan_to_arabic(_str):
    arabian_translation = list()
    for word in _str.split():
        if word.isdigit(): arabian_translation.append(word)
        else:
            arabian_word, word, word_iterator = [], unidecode.unidecode(word.lower()), iter(range(len(word)))
            for i in word_iterator:
                if i < len(word) - 1:
                    if word[i:i+2] in DOUBLE_MOROCCAN_LETTERS:
                        arabian_word.append(_morrocan_letter_to_arabian(word[i:i+2], i, word))
                        next(word_iterator)
                    elif word[i] == word[i+1]:
                        arabian_word.append(_morrocan_letter_to_arabian(word[i], i, word))
                        next(word_iterator)
                    else: arabian_word.append(_morrocan_letter_to_arabian(word[i], i, word))
                else: arabian_word.append(_morrocan_letter_to_arabian(word[i], i, word))
            arabian_translation.append(u''.join(arabian_word).replace(u'\u200e', ''))
    arabian_translation = u' '.join(arabian_translation)
    return arabian_translation

# Transliterate a Arabic letter to an Moroccan letter
def _moroccan_arabic_letter_to_moroccan(letter, position, word): 
    return moroccan_arabic_alphabet[normalize_arabic(de_noise(letter))][0]

# Transliteration from arabic to moroccan
def moroccan_arabic_to_moroccan(_str):
    _str, moroccan_translation = normalize_arabic(de_noise(_str)), list()
    str_iterator = iter(_str.split())
    for word in str_iterator:
        if word.isdigit(): 
            moroccan_translation.append(word)
        else: 
            moroccan_word, word_iterator = [], iter(range(len(word))) 
            for i in word_iterator:
                moroccan_word.append(_moroccan_arabic_letter_to_moroccan(word[i], i, word))
            moroccan_translation.append(''.join(moroccan_word))
    moroccan_translation = ' '.join(moroccan_translation)
    return moroccan_translation